"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("../api");
var fs = require("fs");
var dom_utilities_1 = require("./dom-utilities");
var xmldom_1 = require("xmldom");
/**
 * Created by martin on 05.05.2017.
 * Testcases for XLIFF 2.0 files.
 */
describe('ngx-i18nsupport-lib XLIFF 2.0 test spec', function () {
    var SRCDIR = 'test/testdata/i18n/';
    var ENCODING = 'UTF-8';
    /**
     * Helper function to read Xliff 2.0 from File
     * @type {string}
     */
    function readFile(path) {
        var content = fs.readFileSync(path, ENCODING);
        return api_1.TranslationMessagesFileFactory.fromFileContent('xlf2', content, path, ENCODING);
    }
    describe('XLIFF 2.0 format tests', function () {
        var MASTER1SRC = SRCDIR + 'ngExtractedMaster1.xlf2';
        var TRANSLATED_FILE_SRC = SRCDIR + 'translatedFile.xlf2';
        var ID_APP_RUNS = '4371668001355139802'; // an ID from ngExtractedMaster1.xlf
        var ID_WITH_PLACEHOLDER = '9030312858648510700';
        var ID_WITH_REPEATED_PLACEHOLDER = '7049669989298349710';
        var ID_WITH_MEANING_AND_DESCRIPTION = '6830980354990918030';
        var ID_WITH_NO_SOURCEREFS = '4371668001355139802'; // an ID with no source elements
        var ID_WITH_ONE_SOURCEREF = '7499557905529977371';
        var ID_WITH_TWO_SOURCEREFS = '3274258156935474372'; // an ID with 2 source elements
        var ID_WITH_TAGS = '7609655310648429098';
        var ID_WITH_STRANGE_TAG = '7610784844464920497';
        var ID_TO_MERGE = 'unittomerge';
        var ID_ICU_PLURAL = '157616252019374389';
        var ID_ICU_SELECT = '6710804210857077394';
        var ID_ICU_EMBEDDED_TAGS = '6710804210857077393';
        var ID_CONTAINS_ICU = '2747218257718409559';
        var ID_CONTAINS_TWO_ICU = 'complextags.icuTwoICU';
        var ID_CONTAINS_TWO_ICU_WITH_EQUIV = 'complextags.icuTwoICU.withEquiv'; // angular issue #17344
        var ID_WITH_BR_TAG = '3944017551463298929';
        var ID_WITH_IMG_TAG = '705837031073461246';
        it('should read xlf file', function () {
            var file = readFile(MASTER1SRC);
            expect(file).toBeTruthy();
            expect(file.fileType()).toBe('XLIFF 2.0');
            var tu = file.transUnitWithId(ID_APP_RUNS);
            expect(tu).toBeTruthy();
            expect(tu.sourceContent()).toBe('Anwendung läuft!');
        });
        it('should read xlf2 file and pretty print it and read it in again', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            expect(file).toBeTruthy();
            var formattedXml = file.editedContent(true);
            var rereadFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(formattedXml, null, null);
            expect(rereadFile.numberOfTransUnits()).toBe(file.numberOfTransUnits());
        });
        it('should not add empty lines when beautifying (issue ngx-i18nsupport #97)', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            expect(file).toBeTruthy();
            var editedContentBeautified = file.editedContent(true);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(editedContentBeautified, null, null);
            var editedContentBeautifiedAgain = file2.editedContent(true);
            expect(editedContentBeautifiedAgain).toMatch(/<source>Diese Nachricht ist <pc/);
            expect(editedContentBeautifiedAgain).not.toMatch(/<source>Diese Nachricht ist\s*\r\n?/);
        });
        it('should emit warnings', function () {
            var file = readFile(MASTER1SRC);
            expect(file.warnings().length).toBe(1);
            expect(file.warnings()[0]).toContain('trans-unit without "id"');
        });
        it('should count units', function () {
            var file = readFile(MASTER1SRC);
            expect(file.numberOfTransUnits()).toBe(39);
            expect(file.numberOfTransUnitsWithMissingId()).toBe(1);
            expect(file.numberOfUntranslatedTransUnits()).toBe(file.numberOfTransUnits());
            expect(file.numberOfReviewedTransUnits()).toBe(0);
        });
        it('should return source language', function () {
            var file = readFile(MASTER1SRC);
            expect(file.sourceLanguage()).toBe('de');
        });
        it('should change source language', function () {
            var file = readFile(MASTER1SRC);
            expect(file.sourceLanguage()).toBe('de');
            file.setSourceLanguage('en');
            expect(file.sourceLanguage()).toBe('en');
        });
        it('should return target language', function () {
            var file = readFile(MASTER1SRC);
            expect(file.targetLanguage()).toBeFalsy();
            var translatedFile = readFile(TRANSLATED_FILE_SRC);
            expect(translatedFile.targetLanguage()).toBe('en');
        });
        it('should change target language', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            expect(file.targetLanguage()).toBe('en');
            file.setTargetLanguage('suahel');
            expect(file.targetLanguage()).toBe('suahel');
        });
        it('should loop over all trans units', function () {
            var translatedFile = readFile(TRANSLATED_FILE_SRC);
            var count = 0;
            translatedFile.forEachTransUnit(function (tu) {
                expect(tu).toBeTruthy();
                count++;
            });
            expect(count).toBeGreaterThan(24);
        });
        it('should change source content', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_BR_TAG);
            expect(tu).toBeTruthy();
            expect(tu.supportsSetSourceContent()).toBeTruthy();
            var changedMessage = tu.sourceContent() + 'a changed source content';
            tu.setSourceContent(changedMessage);
            expect(tu.sourceContent()).toBe(changedMessage);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_BR_TAG);
            expect(tu2.sourceContent()).toBe(changedMessage);
        });
        it('should read meaning and description of tu', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu).toBeTruthy();
            expect(tu.meaning()).toBe('dateservice.monday');
            expect(tu.description()).toBe('ngx-translate');
        });
        it('should change description', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu).toBeTruthy();
            expect(tu.description()).toBeTruthy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            var changedMessage = 'a changed description';
            tu.setDescription(changedMessage);
            expect(tu.description()).toBe(changedMessage);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu2.description()).toBe(changedMessage);
        });
        it('should set description (creates new description)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_BR_TAG); // anyone without description
            expect(tu).toBeTruthy();
            expect(tu.description()).toBeFalsy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            var changedMessage = 'a changed description';
            tu.setDescription(changedMessage);
            expect(tu.description()).toBe(changedMessage);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_BR_TAG);
            expect(tu2.description()).toBe(changedMessage);
            var xmlElem = tu2.asXmlElement();
            var notesElem = dom_utilities_1.DOMUtilities.getFirstElementByTagName(xmlElem, 'notes');
            expect(notesElem).toBeTruthy();
        });
        it('should remove description', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu).toBeTruthy();
            expect(tu.description()).toBeTruthy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            tu.setDescription(null);
            expect(tu.description()).toBeFalsy();
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu2.description()).toBeFalsy();
        });
        it('should change meaning', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu).toBeTruthy();
            expect(tu.meaning()).toBeTruthy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            var changedMessage = 'a changed description';
            tu.setMeaning(changedMessage);
            expect(tu.meaning()).toBe(changedMessage);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu2.meaning()).toBe(changedMessage);
        });
        it('should set meaning (creates new meaning)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_BR_TAG); // anyone without description
            expect(tu).toBeTruthy();
            expect(tu.meaning()).toBeFalsy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            var changedMessage = 'a changed description';
            tu.setMeaning(changedMessage);
            expect(tu.meaning()).toBe(changedMessage);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_BR_TAG);
            expect(tu2.meaning()).toBe(changedMessage);
        });
        it('should remove meaning', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu).toBeTruthy();
            expect(tu.meaning()).toBeTruthy();
            expect(tu.supportsSetDescriptionAndMeaning()).toBeTruthy();
            tu.setMeaning(null);
            expect(tu.meaning()).toBeFalsy();
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_MEANING_AND_DESCRIPTION);
            expect(tu2.meaning()).toBeFalsy();
        });
        it('should return empty source references array if source not set', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_NO_SOURCEREFS);
            expect(tu).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(0);
        });
        it('should return source reference', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_ONE_SOURCEREF);
            expect(tu).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(1);
            expect(tu.sourceReferences()[0].sourcefile).toBe('S:/experimente/sampleapp41/src/app/app.component.ts');
            expect(tu.sourceReferences()[0].linenumber).toBe(10);
        });
        it('should return more than one source references', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_TWO_SOURCEREFS);
            expect(tu).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(2);
            expect(tu.sourceReferences()[0].sourcefile).toBe('S:/experimente/sampleapp41/src/app/app.component.ts');
            expect(tu.sourceReferences()[0].linenumber).toBe(20);
            expect(tu.sourceReferences()[1].sourcefile).toBe('S:/experimente/sampleapp41/src/app/app.component.ts');
            expect(tu.sourceReferences()[1].linenumber).toBe(21);
        });
        it('should set source references', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(0);
            tu.setSourceReferences([{ sourcefile: 'x', linenumber: 10 }, { sourcefile: 'y', linenumber: 20 }]);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_TO_MERGE);
            expect(tu2.sourceReferences().length).toBe(2);
            expect(tu2.sourceReferences()[0].sourcefile).toBe('x');
            expect(tu2.sourceReferences()[0].linenumber).toBe(10);
            expect(tu2.sourceReferences()[1].sourcefile).toBe('y');
            expect(tu2.sourceReferences()[1].linenumber).toBe(20);
        });
        it('should override source references', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_TWO_SOURCEREFS);
            expect(tu).toBeTruthy();
            expect(tu.supportsSetSourceReferences()).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(2);
            tu.setSourceReferences([{ sourcefile: 'x:komisch', linenumber: 10 }]);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_TWO_SOURCEREFS);
            expect(tu2.sourceReferences().length).toBe(1);
            expect(tu2.sourceReferences()[0].sourcefile).toBe('x:komisch');
            expect(tu2.sourceReferences()[0].linenumber).toBe(10);
        });
        it('should not change source reference when translating', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_TWO_SOURCEREFS);
            expect(tu).toBeTruthy();
            expect(tu.sourceReferences().length).toBe(2);
            tu.translate('a translated value');
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_TWO_SOURCEREFS);
            expect(tu2.targetContent()).toBe('a translated value');
            expect(tu2.sourceReferences().length).toBe(2);
            expect(tu2.sourceReferences()[0].sourcefile).toBe('S:/experimente/sampleapp41/src/app/app.component.ts');
            expect(tu2.sourceReferences()[0].linenumber).toBe(20);
            expect(tu2.sourceReferences()[1].sourcefile).toBe('S:/experimente/sampleapp41/src/app/app.component.ts');
            expect(tu2.sourceReferences()[1].linenumber).toBe(21);
        });
        it('should run through 3 different states while translating', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_APP_RUNS);
            expect(tu).toBeTruthy();
            expect(tu.targetState()).toBe(api_1.STATE_NEW);
            tu.translate('a translation');
            expect(tu.targetState()).toBe(api_1.STATE_TRANSLATED);
            tu.setTargetState(api_1.STATE_FINAL);
            expect(tu.targetState()).toBe(api_1.STATE_FINAL);
        });
        it('should normalize placeholders to {{0}} etc', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu.targetContentNormalized().asDisplayString()).toBe('Entry {{0}} of total {{1}} added.');
        });
        it('should normalize repeated placeholders to {{0}} {{1}} etc', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            var tu = file.transUnitWithId(ID_WITH_REPEATED_PLACEHOLDER);
            expect(tu.targetContentNormalized().asDisplayString()).toBe('{{0}}: A message with 2 placeholders: {{0}} {{1}}');
        });
        it('should normalize embedded html tags', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            var tu = file.transUnitWithId(ID_WITH_TAGS);
            expect(tu.targetContentNormalized().asDisplayString()).toBe('This message is <b><strong>VERY IMPORTANT</strong></b>');
        });
        it('should normalize unknown embedded html tags', function () {
            var file = readFile(TRANSLATED_FILE_SRC);
            var tu = file.transUnitWithId(ID_WITH_STRANGE_TAG);
            expect(tu.targetContentNormalized().asDisplayString()).toBe('This message is <strange>{{0}}</strange>');
        });
        it('should normalize empty html tag br', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_BR_TAG);
            expect(tu.sourceContentNormalized().asDisplayString()).toBe('Dieser Text enthält<br>einen Zeilenumbruch per HTML-br-Element.');
            var translation = tu.sourceContentNormalized().translate('This text contains<br> a linebreak');
            tu.translate(translation);
            expect(tu.targetContent()).toBe('This text contains<ph id="0" equiv="LINE_BREAK" type="fmt" disp="&lt;br/>"/> a linebreak');
        });
        it('should normalize empty html tag img', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_IMG_TAG);
            expect(tu.sourceContentNormalized().asDisplayString()).toBe('Dieser Text enthält ein Bild <img> mitt en in der Nachricht');
            var translation = tu.sourceContentNormalized().translate('This text contains an img <img> in the message');
            tu.translate(translation);
            expect(tu.targetContent()).toBe('This text contains an img <ph id="0" equiv="TAG_IMG" type="image" disp="&lt;img/>"/> in the message');
        });
        it('should remove a transunit by id', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu).toBeTruthy();
            file.removeTransUnitWithId(ID_WITH_PLACEHOLDER);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu2).toBeFalsy(); // should not exist any more
        });
        it('should translate source without or with target', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_APP_RUNS);
            expect(tu).toBeTruthy();
            expect(tu.targetContent()).toBeFalsy();
            // first translate
            tu.translate('Anwendung läuft');
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_APP_RUNS);
            expect(tu2.targetContentNormalized().asDisplayString()).toBe('Anwendung läuft');
            expect(tu2.targetState()).toBe(api_1.STATE_TRANSLATED);
            // translate again
            tu2.translate('Anwendung funktioniert');
            var file3 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file2.editedContent(), null, null);
            var tu3 = file3.transUnitWithId(ID_APP_RUNS);
            expect(tu3.targetContentNormalized().asDisplayString()).toBe('Anwendung funktioniert');
            expect(tu3.targetState()).toBe(api_1.STATE_TRANSLATED);
        });
        it('should copy source to target for default lang', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu).toBeTruthy();
            expect(tu.targetContent()).toBeFalsy();
            var isDefaultLang = true;
            var copyContent = false;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tu2 = file2.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu2.targetContentNormalized().asDisplayString()).toBe('Eintrag {{0}} von {{1}} hinzugefügt.');
        });
        it('should copy source to target for non default lang if wanted', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu).toBeTruthy();
            expect(tu.targetContent()).toBeFalsy();
            var isDefaultLang = false;
            var copyContent = true;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tu2 = file2.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu2.targetContentNormalized().asDisplayString()).toBe('Eintrag {{0}} von {{1}} hinzugefügt.');
        });
        it('should copy source to target for non default lang and set state new, native state initial, #57', function () {
            var file = readFile(MASTER1SRC);
            var isDefaultLang = false;
            var copyContent = true;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tu2 = file2.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu2.targetContentNormalized().asDisplayString()).toBe('Eintrag {{0}} von {{1}} hinzugefügt.');
            expect(tu2.targetState()).toBe(api_1.STATE_NEW);
            expect(tu2.nativeTargetState()).toBe('initial');
        });
        it('should not copy source to target for non default lang if not wanted', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu).toBeTruthy();
            expect(tu.targetContent()).toBeFalsy();
            var isDefaultLang = false;
            var copyContent = false;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tu2 = file2.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu2.targetContent()).toBeFalsy();
        });
        it('should copy a transunit from file a to file b', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeFalsy();
            var newTu = targetFile.importNewTransUnit(tu, false, true);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeTruthy();
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toEqual(newTu);
            var changedTargetFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(targetFile.editedContent(), null, null);
            var targetTu = changedTargetFile.transUnitWithId(ID_TO_MERGE);
            expect(targetTu.sourceContent()).toBe('Test for merging units');
        });
        it('should copy a transunit to a specified position (#53)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeFalsy();
            var ID_EXISTING = '7499557905529977371';
            var existingTu = targetFile.transUnitWithId(ID_EXISTING);
            expect(existingTu).toBeTruthy();
            var newTu = targetFile.importNewTransUnit(tu, false, true, existingTu);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeTruthy();
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toEqual(newTu);
            var doc = new xmldom_1.DOMParser().parseFromString(targetFile.editedContent());
            var existingElem = dom_utilities_1.DOMUtilities.getElementByTagNameAndId(doc, 'unit', ID_EXISTING);
            var newElem = dom_utilities_1.DOMUtilities.getElementByTagNameAndId(doc, 'unit', ID_TO_MERGE);
            expect(dom_utilities_1.DOMUtilities.getElementFollowingSibling(existingElem)).toEqual(newElem);
            var changedTargetFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(targetFile.editedContent(), null, null);
            var targetTu = changedTargetFile.transUnitWithId(ID_TO_MERGE);
            expect(targetTu.sourceContent()).toBe('Test for merging units');
        });
        it('should copy a transunit to first position (#53)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeFalsy();
            // when importNewTransUnit is called with null, new unit will be added at first position
            var newTu = targetFile.importNewTransUnit(tu, false, true, null);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeTruthy();
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toEqual(newTu);
            var doc = new xmldom_1.DOMParser().parseFromString(targetFile.editedContent());
            var newElem = dom_utilities_1.DOMUtilities.getElementByTagNameAndId(doc, 'unit', ID_TO_MERGE);
            expect(newElem).toBeTruthy();
            expect(dom_utilities_1.DOMUtilities.getElementPrecedingSibling(newElem)).toBeFalsy();
            var changedTargetFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(targetFile.editedContent(), null, null);
            var targetTu = changedTargetFile.transUnitWithId(ID_TO_MERGE);
            expect(targetTu.sourceContent()).toBe('Test for merging units');
        });
        it('should copy source to target and set a praefix and suffix', function () {
            var file = readFile(MASTER1SRC);
            file.setNewTransUnitTargetPraefix('%%');
            file.setNewTransUnitTargetSuffix('!!');
            var isDefaultLang = false;
            var copyContent = true;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tu2 = file2.transUnitWithId(ID_TO_MERGE);
            expect(tu2.targetState()).toBe(api_1.STATE_NEW);
            expect(tu2.targetContent()).toBe('%%Test for merging units!!');
        });
        it('should copy source to target, but should not set a praefix and suffix for ICU messages', function () {
            var file = readFile(MASTER1SRC);
            file.setNewTransUnitTargetPraefix('%%');
            file.setNewTransUnitTargetSuffix('!!');
            var isDefaultLang = false;
            var copyContent = true;
            var file2 = file.createTranslationFileForLang('xy', null, isDefaultLang, copyContent);
            var tuICU = file2.transUnitWithId(ID_ICU_PLURAL);
            expect(tuICU.targetState()).toBe(api_1.STATE_NEW);
            expect(tuICU.targetContent()).not.toContain('%%');
            expect(tuICU.targetContent()).not.toContain('!!');
        });
        it('should copy a transunit from file a to file b and set a praefix and suffix', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            targetFile.setNewTransUnitTargetPraefix('%%');
            targetFile.setNewTransUnitTargetSuffix('!!');
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeFalsy();
            var newTu = targetFile.importNewTransUnit(tu, false, true);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeTruthy();
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toEqual(newTu);
            var changedTargetFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(targetFile.editedContent(), null, null);
            var targetTu = changedTargetFile.transUnitWithId(ID_TO_MERGE);
            expect(targetTu.targetContent()).toBe('%%Test for merging units!!');
        });
        it('should copy a transunit from file a to file b, but should not set a praefix and suffix for ICU messages', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_SELECT);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            targetFile.setNewTransUnitTargetPraefix('%%');
            targetFile.setNewTransUnitTargetSuffix('!!');
            expect(targetFile.transUnitWithId(ID_ICU_SELECT)).toBeFalsy();
            var newTu = targetFile.importNewTransUnit(tu, false, true);
            expect(targetFile.transUnitWithId(ID_ICU_SELECT)).toBeTruthy();
            expect(targetFile.transUnitWithId(ID_ICU_SELECT)).toEqual(newTu);
            var changedTargetFile = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(targetFile.editedContent(), null, null);
            var targetTu = changedTargetFile.transUnitWithId(ID_ICU_SELECT);
            expect(targetTu.targetContent()).not.toContain('%%');
            expect(targetTu.targetContent()).not.toContain('!!');
        });
        it('should preserve line end at end of file while editing', function () {
            var content = fs.readFileSync(MASTER1SRC, ENCODING);
            expect(content.endsWith('\n')).toBeTruthy('Master should end with EOL');
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_TO_MERGE);
            expect(tu).toBeTruthy();
            var targetFile = readFile(TRANSLATED_FILE_SRC);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeFalsy();
            targetFile.importNewTransUnit(tu, false, true);
            expect(targetFile.transUnitWithId(ID_TO_MERGE)).toBeTruthy();
            expect(targetFile.editedContent().endsWith('\n')).toBeTruthy('Edited content should end with EOL');
        });
        it('should translate using NormalizedMessage (plain text case, no placeholders, no markup)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_APP_RUNS);
            expect(tu).toBeTruthy();
            var translationString = 'Anwendung läuft';
            // first translate
            var translation = tu.sourceContentNormalized().translate(translationString);
            tu.translate(translation);
            expect(tu.targetContent()).toBe(translationString);
            var file2 = api_1.TranslationMessagesFileFactory.fromUnknownFormatFileContent(file.editedContent(), null, null);
            var tu2 = file2.transUnitWithId(ID_APP_RUNS);
            expect(tu2.targetContentNormalized().asDisplayString()).toBe(translationString);
        });
        it('should translate placeholders without loosing disp info', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_WITH_PLACEHOLDER);
            expect(tu).toBeTruthy();
            var normalizedMessage = tu.sourceContentNormalized();
            // Eintrag <ph id="0" equiv="INTERPOLATION" disp="{{number()}}"/> von <ph id="1" equiv="INTERPOLATION_1" disp="{{total()}}"/> hinzugefügt.
            expect(normalizedMessage.asDisplayString()).toBe('Eintrag {{0}} von {{1}} hinzugefügt.');
            var translatedMessage = normalizedMessage.translate('Total {{1}}, added {{0}}');
            tu.translate(translatedMessage);
            expect(tu.targetContent()).toBe('Total <ph id="0" equiv="INTERPOLATION_1" disp="{{total()}}"/>, added <ph id="1" equiv="INTERPOLATION" disp="{{number()}}"/>');
        });
        it('should contain ICU reference in sourceContentNormalized', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_CONTAINS_ICU);
            expect(tu).toBeTruthy();
            expect(tu.sourceContent()).toBe('Zum Wert <ph id="0" equiv="INTERPOLATION" disp="{{auswahl}}"/> gehört der Text <ph id="1"/>');
            expect(tu.sourceContentNormalized().asDisplayString()).toBe('Zum Wert {{0}} gehört der Text <ICU-Message-Ref_1/>');
        });
        it('should contain 2 ICU references in sourceContentNormalized (old syntax before angular #17344)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_CONTAINS_TWO_ICU);
            expect(tu).toBeTruthy();
            expect(tu.sourceContent()).toBe('first: <ph id="0"/>, second <ph id="1"/>');
            expect(tu.sourceContentNormalized().asDisplayString()).toBe('first: <ICU-Message-Ref_0/>, second <ICU-Message-Ref_1/>');
        });
        it('should contain 2 ICU references in sourceContentNormalized (new syntax after angular #17344)', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_CONTAINS_TWO_ICU_WITH_EQUIV);
            expect(tu).toBeTruthy();
            expect(tu.sourceContent()).toBe('first: <ph id="0" equiv="ICU" disp="{count, plural, =0 {...} =1 {...} other {...}}"/>, second <ph id="1" equiv="ICU_1" disp="{gender, select, m {...} f {...}}"/>');
            expect(tu.sourceContentNormalized().asDisplayString()).toBe('first: <ICU-Message-Ref_0/>, second <ICU-Message-Ref_1/>');
        });
        it('should translate ICU references without loosing disp info', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_CONTAINS_TWO_ICU_WITH_EQUIV);
            expect(tu).toBeTruthy();
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('first: <ICU-Message-Ref_0/>, second <ICU-Message-Ref_1/>');
            var translatedMessage = normalizedMessage.translate('Zweitens <ICU-Message-Ref_1/>, Erstens: <ICU-Message-Ref_0/>');
            tu.translate(translatedMessage);
            expect(tu.targetContent()).toBe('Zweitens <ph id="1" equiv="ICU_1" disp="{gender, select, m {...} f {...}}"/>, Erstens: <ph id="0" equiv="ICU" disp="{count, plural, =0 {...} =1 {...} other {...}}"/>');
        });
        it('should handle plural ICU', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_PLURAL);
            expect(tu).toBeTruthy();
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('<ICU-Message/>');
            var icuMessage = normalizedMessage.getICUMessage();
            expect(icuMessage).toBeTruthy();
            expect(icuMessage.isPluralMessage()).toBeTruthy();
            expect(icuMessage.isSelectMessage()).toBeFalsy();
            expect(icuMessage.getCategories().length).toBe(3);
            expect(icuMessage.getCategories()[0].getCategory()).toBe('=0');
            expect(icuMessage.getCategories()[0].getMessageNormalized().asDisplayString()).toBe('kein Schaf');
            expect(icuMessage.getCategories()[1].getCategory()).toBe('=1');
            expect(icuMessage.getCategories()[1].getMessageNormalized().asDisplayString()).toBe('1 Schaf');
            expect(icuMessage.getCategories()[2].getCategory()).toBe('other');
            expect(icuMessage.getCategories()[2].getMessageNormalized().asDisplayString()).toBe('x Schafe');
        });
        it('should translate plural ICU', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_PLURAL);
            expect(tu).toBeTruthy();
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('<ICU-Message/>');
            var translatedMessage = normalizedMessage.translateICUMessage({ '=0': 'nothing' });
            tu.translate(translatedMessage);
            var icuMessage = tu.targetContentNormalized().getICUMessage();
            expect(icuMessage).toBeTruthy();
            expect(icuMessage.isPluralMessage()).toBeTruthy();
            expect(icuMessage.isSelectMessage()).toBeFalsy();
            expect(icuMessage.getCategories().length).toBe(3);
            expect(icuMessage.getCategories()[0].getCategory()).toBe('=0');
            expect(icuMessage.getCategories()[0].getMessageNormalized().asDisplayString()).toBe('nothing');
            expect(tu.targetContent()).toBe('{VAR_PLURAL, plural, =0 {nothing} =1 {1 Schaf} other {x Schafe}}');
        });
        it('should handle select ICU message', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_SELECT);
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('<ICU-Message/>');
            var icuMessage = normalizedMessage.getICUMessage();
            expect(icuMessage).toBeTruthy();
            expect(icuMessage.isPluralMessage()).toBeFalsy();
            expect(icuMessage.isSelectMessage()).toBeTruthy();
            expect(icuMessage.getCategories().length).toBe(3);
            expect(icuMessage.getCategories()[0].getCategory()).toBe('wert0');
            expect(icuMessage.getCategories()[0].getMessageNormalized().asDisplayString()).toBe('wert0 wurde gewählt');
            expect(icuMessage.getCategories()[1].getCategory()).toBe('wert1');
            expect(icuMessage.getCategories()[1].getMessageNormalized().asDisplayString()).toBe('ein anderer Wert (wert1) wurde gewählt');
            expect(icuMessage.getCategories()[2].getCategory()).toBe('wert2');
            expect(icuMessage.getCategories()[2].getMessageNormalized().asDisplayString()).toBe('was ganz anderes wurde gewählt');
        });
        it('should handle ICU with embedded tags', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_EMBEDDED_TAGS);
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('<ICU-Message/>');
            var icuMessage = normalizedMessage.getICUMessage();
            expect(icuMessage).toBeTruthy();
            expect(icuMessage.isPluralMessage()).toBeFalsy();
            expect(icuMessage.isSelectMessage()).toBeTruthy();
            expect(icuMessage.getCategories().length).toBe(3);
            expect(icuMessage.getCategories()[0].getCategory()).toBe('wert0');
            expect(icuMessage.getCategories()[0].getMessageNormalized().asDisplayString()).toBe('wert0 ausgewählt');
            expect(icuMessage.getCategories()[1].getCategory()).toBe('wert1');
            expect(icuMessage.getCategories()[1].getMessageNormalized().asDisplayString()).toBe('ein <b>anderer</b> Wert (wert1) ausgewählt');
            expect(icuMessage.getCategories()[2].getCategory()).toBe('wert2');
            expect(icuMessage.getCategories()[2].getMessageNormalized().asDisplayString()).toBe('was <em>ganz anderes</em> wurde ausgewählt');
        });
        it('should translate ICU with embedded tags', function () {
            var file = readFile(MASTER1SRC);
            var tu = file.transUnitWithId(ID_ICU_EMBEDDED_TAGS);
            var normalizedMessage = tu.sourceContentNormalized();
            expect(normalizedMessage.asDisplayString()).toBe('<ICU-Message/>');
            var translatedMessage = normalizedMessage.translateICUMessage({ 'wert1': '<em>changed</em>' });
            tu.translate(translatedMessage);
            var icuMessage = tu.targetContentNormalized().getICUMessage();
            expect(icuMessage).toBeTruthy();
            expect(icuMessage.isPluralMessage()).toBeFalsy();
            expect(icuMessage.isSelectMessage()).toBeTruthy();
            expect(icuMessage.getCategories().length).toBe(3);
            expect(icuMessage.getCategories()[1].getCategory()).toBe('wert1');
            expect(icuMessage.getCategories()[1].getMessageNormalized().asDisplayString()).toBe('<em>changed</em>');
            expect(tu.targetContent()).toContain('wert1 {<pc id="0" equivStart="START_EMPHASISED_TEXT" equivEnd="CLOSE_EMPHASISED_TEXT" type="other" dispStart="&lt;em>" dispEnd="&lt;/em>">changed</pc>}');
            // TODO find warnings in embedded message, known limitation in the moment.
            //            const warnings = icuMessage.getCategories()[1].getMessageNormalized().validateWarnings();
            //            expect(warnings).toBeTruthy();
        });
    });
});
//# sourceMappingURL=S:/experimente/ngx-i18nsupport-lib/src/impl/xliff2-file.spec.js.map